
@class YHNode;
@class YHTopNode;
@class YHToolbarView;
@class YHWebControlView;
@class YHStacksKitPlugin;
@class YHNodeTextEditView;

@protocol YHStacksImageSaving;

@interface YHMainViewController : NSObject {
	YHTopNode                   *_root;
	
    WebView                     *_webView;
    YHWebControlView            *_controlView;
    YHStacksKitPlugin           *_plugin;
    id <YHStacksImageSaving>    _sandwichController;
    YHToolbarView               *_toolbar;
	
	NSPoint                     _savedScrollPoint;
    NSOperationQueue            *_bgQueueStorage;
	
	BOOL                        loading;
	BOOL                        protocolRegistered;
  
    YHNode                      *_selection;
    YHNode                      *_pick;
    BOOL                        _validShown;
    
    YHNode                      *_saveNode;
    NSWindow                    *_saveWindow;
    NSTextField                 *_saveTitleTextField;
    NSString                    *_saveTitleString;
    NSString                    *_saveSubtitleString;
    NSString                    *_saveDescriptionString;
    NSString                    *_saveGroupString;
//    BOOL                        _saveShouldClearText;
    BOOL                        _saveShouldClearImages;
    BOOL                        _saveShouldLock;
    BOOL                        _saveButtonEnabled;
}

+ (void)addController:(id)newController withUid:(NSString *)uid;
+ (void)removeControllerWithUid:(NSString *)uid;
+ (id)controllerWithUid:(NSString *)uid;
+ (NSArray *)allControllers;

- (void)moveNode:(YHNode *)node before:(YHNode *)sibling;
- (void)moveNode:(YHNode *)node after:(YHNode *)sibling;
- (void)moveNode:(YHNode *)node toParent:(YHNode *)parent;
- (YHNode *)duplicateNode:(YHNode *)node before:(YHNode *)sibling;
- (YHNode *)duplicateNode:(YHNode *)node after:(YHNode *)sibling;
- (YHNode *)duplicateNode:(YHNode *)node toParent:(YHNode *)parent;
- (void)addNode:(YHNode *)node before:(YHNode *)sibling;
- (void)addNode:(YHNode *)node after:(YHNode *)sibling;
- (void)addNode:(YHNode *)node toParent:(YHNode *)parent;

//- (void)setSelectedObject:(id)object;
//- (YHNode *)selectedNode;
- (void)showValidator;

- (void)clearAll;
- (YHNode *)nodeAtPoint:(NSPoint)location;
- (YHNode *)sliceAtPoint:(NSPoint)location;
- (YHNode *)sliceOfNode:(YHNode *)node;
- (DOMNode *)DOMNodeAtPoint:(NSPoint)location;
- (DOMNode *)DOMNodeOfSliceAtPoint:(NSPoint)location;
- (YHNode *)nodeForDOMNode:(DOMNode *)domNode;
- (YHNode *)firstFreeContainerOfNode:(YHNode *)node;
- (YHNode *)firstFreeContainerAtPoint:(NSPoint)location;
- (YHNode *)replaceableNodeAtPoint:(NSPoint)location;
- (YHNode *)childOfNode:(YHNode *)parent atPoint:(NSPoint)location;
- (YHNode *)draggableParentOf:(YHNode *)childNode;
- (NSRect)boxForNode:(YHNode *)node;

- (YHNode *)nodeWithUid:(NSString *)uid;

- (YHNode *)selectableContainerOf:(YHNode *)node;
- (YHNode *)selectableParentOf:(YHNode *)node;
- (YHNode *)selectableChildOf:(YHNode *)node;
- (YHNode *)selectableNextSiblingOf:(YHNode *)node;
- (YHNode *)selectablePrevSiblingOf:(YHNode *)node;

- (void)selectLeft;
- (void)selectRight;
- (void)selectUp;
- (void)selectDown;
- (void)selectIn;
- (void)selectOut;

- (DOMHTMLElement *)domNodeForElementId:(NSString *)elementId;
- (DOMHTMLElement *)domNodeForNode:(YHNode *)node;
- (DOMHTMLElement *)outerDOMNodeForNode:(YHNode *)node;
- (DOMHTMLElement *)mezDOMNodeForNode:(YHNode *)node;

- (DOMNode *)rawDOMNodeAtPoint:(NSPoint)location;
- (DOMNode *)parentOfNode:(DOMNode *)domNode withProperty:(NSString *)property value:(NSString *)value;

- (void)makeChildrenOf:(YHNode *)parent performSelector:(SEL)aSelector;
- (void)makeAllChildrenPerformSelector:(SEL)aSelector;

- (void)updateWebView;
- (IBAction)update:(id)sender;

- (IBAction)doStopEditing:(id)sender;
- (IBAction)doStartEditing:(id)sender;

- (void) displayChildrenOf:(YHNode *)parent;
- (void)flush;
- (void)refresh;
- (void)display;

- (void)populateWithNewRootNode:(YHTopNode *)newRoot;

- (IBAction)doNodeHelp:(id)sender;
- (IBAction)doNodeInfo:(id)sender;
- (IBAction)doUpdateNow:(id)sender;

- (IBAction)doSaveStack:(id)sender;

- (NSString *)previewHTML;
- (NSString *)plist;

- (void)saveUndoStateWithNode:(YHNode *)node keyPath:(NSString *)keyPath title:(NSString *)title;
- (void)saveUndoStateForAdd:(YHNode *)node;
- (void)saveUndoStateForMove:(YHNode *)node;
- (void)saveUndoStateForRemove:(YHNode *)node;

- (IBAction)doSaveSaveButton:(id)sender;
- (IBAction)doSaveCancelButton:(id)sender;
- (IBAction)doSaveHelpButton:(id)sender;

@property (assign)		YHTopNode                       *root;
@property (assign)		IBOutlet YHStacksKitPlugin      *plugin;
@property (readonly)      id <YHStacksImageSaving>        sandwichController;

@property (assign)		IBOutlet WebView                *webView;
@property (assign)		IBOutlet YHWebControlView       *controlView;
@property (assign)		IBOutlet YHToolbarView          *toolbar;

@property (assign)      YHNode                          *selection;
@property (assign)      YHNode                          *pick;

@property (readonly)    YHNodeTextEditView              *textEditView;
@property (assign)      NSPoint                         savedScrollPoint;
@property (assign)      BOOL                            validShown;

@property (retain)      YHNode                  *saveNode;
@property (assign)		IBOutlet NSWindow       *saveWindow;
@property (assign)      IBOutlet NSTextField    *saveTitleTextField;
@property (retain)      NSString    *saveTitleString;
@property (retain)      NSString    *saveSubtitleString;
@property (retain)      NSString    *saveDescriptionString;
@property (retain)      NSString    *saveGroupString;
//@property (assign)      BOOL        saveShouldClearText;
@property (assign)      BOOL        saveShouldClearImages;
@property (assign)      BOOL        saveShouldLock;
@property (assign, readonly)      BOOL        saveButtonEnabled;


@end
